@@ -24,10 +24,8 @@
 #include <config.h>
 #endif
 
-#include "public/ds_sds_session.h"
 #include "public/scap_ds.h"
-#include "public/xccdf_benchmark.h"
-#include "public/oval_definitions.h"
+
 #include "public/oscap.h"
 #include "public/oscap_text.h"
 
@@ -791,7 +789,14 @@ static int ds_sds_compose_catalog_has_uri(xmlDocPtr doc, xmlNodePtr catalog, con
 	xpathCtx->node = catalog;
 
 	char *expression = oscap_sprintf("cat:uri[@uri = '%s']", uri);
+	if (expression == NULL)
+	{
+		oscap_seterr(OSCAP_EFAMILY_OSCAP, "Error: Unable to create XPath expression.");
+		xmlXPathFreeContext(xpathCtx);
 
+		return -1;
+	}
+	
 	xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression(
 			BAD_CAST expression,
 			xpathCtx);
@@ -1012,6 +1017,13 @@ static int ds_sds_compose_has_component_ref(xmlDocPtr doc, xmlNodePtr datastream
 	xpathCtx->node = datastream;
 
 	char *expression = oscap_sprintf("*/ds:component-ref[@xlink:href = '#%s' and @id = '%s']", filepath, cref_id);
+	if (expression == NULL)
+	{
+		oscap_seterr(OSCAP_EFAMILY_OSCAP, "Error: Unable to create XPath expression.");
+		xmlXPathFreeContext(xpathCtx);
+
+		return -1;
+	}
 
 	xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression(
 			BAD_CAST expression,
