@@ -20,6 +20,8 @@
  *      Peter Vrabec   <pvrabec@redhat.com>
  */
 
+#include "oscap_string.h"
+#include "oscap_text.h"
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -90,6 +92,7 @@ struct oscap_module OSCAP_XCCDF_MODULE = {
     .parent = &OSCAP_ROOT_MODULE,
     .summary = "eXtensible Configuration Checklist Description Format",
     .usage_extra = "command [command-specific-options]",
+    .opt_parser = getopt_xccdf,
     .submodules = XCCDF_SUBMODULES
 };
 
@@ -100,7 +103,10 @@ static struct oscap_module XCCDF_RESOLVE = {
     .usage = "[options] -o output-xccdf.xml input-xccdf.xml",
     .help =
 		"Options:\n"
-		"   --force                       - Force resolving XCCDF document even if it is aleready marked as resolved.",
+		"   --force                       - Force resolving XCCDF document even if it is aleready marked as resolved.\n"
+		"   --verbose <verbosity_level>   - Turn on verbose mode at specified verbosity level.\n"
+		"                                   Verbosity level must be one of: DEVEL, INFO, WARNING, ERROR.\n"
+		"   --verbose-log-file <file>     - Write verbose information into file.\n",
     .opt_parser = getopt_xccdf,
     .func = app_xccdf_resolve
 };
@@ -114,7 +120,9 @@ static struct oscap_module XCCDF_VALIDATE = {
 	.func = app_xccdf_validate,
 	.help = "Options:\n"
 		"   --skip-schematron             - Do not use schematron-based validation in addition to XML Schema\n"
-	,
+		"   --verbose <verbosity_level>   - Turn on verbose mode at specified verbosity level.\n"
+		"                                   Verbosity level must be one of: DEVEL, INFO, WARNING, ERROR.\n"
+		"   --verbose-log-file <file>     - Write verbose information into file.\n",
 };
 
 static struct oscap_module XCCDF_EXPORT_OVAL_VARIABLES = {
@@ -138,7 +146,9 @@ static struct oscap_module XCCDF_EXPORT_OVAL_VARIABLES = {
 		"                                   (only applicable when datastream-id AND xccdf-id are not specified)\n"
 		"   --cpe <name>                  - Use given CPE dictionary or language (autodetected)\n"
 		"                                   for applicability checks.\n"
-	,
+		"   --verbose <verbosity_level>   - Turn on verbose mode at specified verbosity level.\n"
+		"                                   Verbosity level must be one of: DEVEL, INFO, WARNING, ERROR.\n"
+		"   --verbose-log-file <file>     - Write verbose information into file.\n",
 };
 
 static struct oscap_module XCCDF_EVAL = {
@@ -185,7 +195,11 @@ static struct oscap_module XCCDF_EVAL = {
 		"                                   (only applicable for source data streams)\n"
 		"                                   (only applicable when datastream-id AND xccdf-id are not specified)\n"
 		"   --remediate                   - Automatically execute XCCDF fix elements for failed rules.\n"
-		"                                   Use of this option is always at your own risk.\n",
+		"                                   Use of this option is always at your own risk.\n"
+		"   --verbose <verbosity_level>   - Turn on verbose mode at specified verbosity level.\n"
+		"                                   Verbosity level must be one of: DEVEL, INFO, WARNING, ERROR.\n"
+		"   --verbose-log-file <file>     - Write verbose information into file.\n"
+		"   --show-rule-details           - Show rule description and rationale when reporting results.\n",
     .opt_parser = getopt_xccdf,
     .func = app_evaluate_xccdf
 };
@@ -214,7 +228,9 @@ static struct oscap_module XCCDF_REMEDIATE = {
 		"                                   Format is \"$rule_id:$result\\n\".\n"
 		"   --progress-full               - Switch to sparse but a bit more saturated output also suitable for progress reporting.\n"
 		"                                   Format is \"$rule_id|$rule_title|$result\\n\".\n"
-	,
+		"   --verbose <verbosity_level>   - Turn on verbose mode at specified verbosity level.\n"
+		"                                   Verbosity level must be one of: DEVEL, INFO, WARNING, ERROR.\n"
+		"   --verbose-log-file <file>     - Write verbose information into file.\n",
 	.opt_parser = getopt_xccdf,
 	.func = app_xccdf_remediate
 };
@@ -242,7 +258,10 @@ static struct oscap_module XCCDF_GEN_REPORT = {
     .help = GEN_OPTS
 		"\nReport Options:\n"
 		"   --result-id <id>              - TestResult ID to be processed. Default is the most recent one.\n"
-		"   --output <file>               - Write the document into file.\n",
+		"   --output <file>               - Write the document into file.\n"
+		"   --verbose <verbosity_level>   - Turn on verbose mode at specified verbosity level.\n"
+		"                                   Verbosity level must be one of: DEVEL, INFO, WARNING, ERROR.\n"
+		"   --verbose-log-file <file>     - Write verbose information into file.\n",
     .opt_parser = getopt_xccdf,
     .user = "xccdf-report.xsl",
     .func = app_xccdf_xslt
@@ -267,7 +286,10 @@ static struct oscap_module XCCDF_GEN_GUIDE = {
 		"                                   (only applicable for source data streams)\n"
 		"   --skip-signature-validation   - Skip data stream signature validation.\n"
 		"                                   (only applicable for source data streams)\n"
-		"   --enforce-signature           - Process only signed data streams.\n",
+		"   --enforce-signature           - Process only signed data streams.\n"
+		"   --verbose <verbosity_level>   - Turn on verbose mode at specified verbosity level.\n"
+		"                                   Verbosity level must be one of: DEVEL, INFO, WARNING, ERROR.\n"
+		"   --verbose-log-file <file>     - Write verbose information into file.\n",
     .opt_parser = getopt_xccdf,
     .user = NULL,
     .func = app_generate_guide
@@ -295,7 +317,10 @@ static struct oscap_module XCCDF_GEN_FIX = {
 		"                                   (only applicable for source data streams)\n"
 		"   --skip-signature-validation   - Skip data stream signature validation.\n"
 		"                                   (only applicable for source data streams)\n"
-		"   --enforce-signature           - Process only signed data streams.\n",
+		"   --enforce-signature           - Process only signed data streams.\n"
+		"   --verbose <verbosity_level>   - Turn on verbose mode at specified verbosity level.\n"
+		"                                   Verbosity level must be one of: DEVEL, INFO, WARNING, ERROR.\n"
+		"   --verbose-log-file <file>     - Write verbose information into file.\n",
     .opt_parser = getopt_xccdf,
     .user = "legacy-fix.xsl",
     .func = app_generate_fix
@@ -309,7 +334,10 @@ static struct oscap_module XCCDF_GEN_CUSTOM = {
     .help = GEN_OPTS
 		"\nCustom Options:\n"
 		"   --stylesheet <file>           - Specify an absolute path to a custom stylesheet to format the output.\n"
-		"   --output <file>               - Write the document into file.\n",
+		"   --output <file>               - Write the document into file.\n"
+		"   --verbose <verbosity_level>   - Turn on verbose mode at specified verbosity level.\n"
+		"                                   Verbosity level must be one of: DEVEL, INFO, WARNING, ERROR.\n"
+		"   --verbose-log-file <file>     - Write verbose information into file.\n",
     .opt_parser = getopt_xccdf,
     .user = NULL,
     .func = app_xccdf_xslt
@@ -352,16 +380,48 @@ static const char * RESULT_COLORS[] = {"", "32", "31", "1;31", "1;30", "1", "1",
 
 static char custom_stylesheet_path[PATH_MAX];
 
+static void _print_rule_details(struct xccdf_policy *policy, struct xccdf_item *item)
+{
+	if (!xccdf_policy_get_show_rule_details(policy)) {
+		return;
+	}
+	char *description = xccdf_policy_get_readable_item_description(policy, item, NULL);
+	if (strlen(description) > 0) {
+		char *indented_description = oscap_indent(description, 8);
+		printf("Description\n%s\n", indented_description);
+		free(indented_description);
+	}
+	free(description);
+	char *rationale = xccdf_policy_get_readable_item_rationale(policy, item, NULL);
+	if (strlen(rationale) > 0) {
+		char *indented_rationale = oscap_indent(rationale, 8);
+		printf("Rationale\n%s\n", indented_rationale);
+		free(indented_rationale);
+	}
+	free(rationale);
+	struct oscap_stringlist *warnings = xccdf_rule_get_warnings_strings((struct xccdf_rule *) item);
+	struct oscap_string_iterator *warnings_it = oscap_stringlist_get_strings(warnings);
+	while (oscap_string_iterator_has_more(warnings_it)) {
+		const char *warning = oscap_string_iterator_next(warnings_it);
+		char *indented_warning = oscap_indent(warning, 8);
+		printf("Warning\n%s\n", indented_warning);
+		free(indented_warning);
+	}
+	oscap_string_iterator_free(warnings_it);
+	oscap_stringlist_free(warnings);
+}
+
 static int callback_scr_rule(struct xccdf_rule *rule, void *arg)
 {
+	struct xccdf_policy *policy = (struct xccdf_policy *) arg;
 	const char * rule_id = xccdf_rule_get_id(rule);
 
 	/* is rule selected? we print only selected rules */
-	const bool selected = xccdf_policy_is_item_selected((struct xccdf_policy *) arg, rule_id);
+	const bool selected = xccdf_policy_is_item_selected(policy, rule_id);
 	if (!selected)
 		return 0;
 
-	const char *title = xccdf_policy_get_readable_item_title((struct xccdf_policy *)arg, (struct xccdf_item *) rule, NULL);
+	const char *title = xccdf_policy_get_readable_item_title(policy, (struct xccdf_item *) rule, NULL);
 
 	/* print */
 	if (isatty(1)) {
@@ -383,6 +443,7 @@ static int callback_scr_rule(struct xccdf_rule *rule, void *arg)
 #else
 	printf("Rule\r\t%s\n", rule_id);
 #endif
+	_print_rule_details(policy, (struct xccdf_item *) rule);
 
 	struct xccdf_ident_iterator *idents = xccdf_rule_get_idents(rule);
 	while (xccdf_ident_iterator_has_more(idents)) {
@@ -582,6 +643,49 @@ int xccdf_set_profile_or_report_bad_id(struct xccdf_session *session, const char
 	return return_code;
 }
 
+
+static bool _system_is_in_bootc_mode(void)
+{
+#ifdef OS_WINDOWS
+	return false;
+#else
+	#define BOOTC_PATH "/usr/bin/bootc"
+	#define CHUNK_SIZE 1024
+	struct stat statbuf;
+	if (stat(BOOTC_PATH, &statbuf) == -1) {
+		return false;
+	}
+	FILE *output = popen(BOOTC_PATH " status --format json 2>/dev/null", "r");
+	if (output == NULL) {
+		return false;
+	}
+	size_t buf_size = CHUNK_SIZE;
+	char *buf = calloc(buf_size, sizeof(char));
+	if (buf == NULL) {
+		pclose(output);
+		return false;
+	}
+	int c;
+	size_t i = 0;
+	while ((c = fgetc(output)) != EOF) {
+		if (i >= buf_size) {
+			buf_size += CHUNK_SIZE;
+			char *new_buf = realloc(buf, buf_size);
+			if (new_buf == NULL) {
+				pclose(output);
+				return false;
+			}
+			buf = new_buf;
+		}
+		buf[i++] = c;
+	}
+	pclose(output);
+	bool result = (*buf != '\0' && strstr(buf, "\"booted\":null") == NULL);
+	free(buf);
+	return result;
+#endif
+}
+
 /**
  * XCCDF Processing fucntion
  * @param action OSCAP Action structure
@@ -592,6 +696,16 @@ int app_evaluate_xccdf(const struct oscap_action *action)
 	struct xccdf_session *session = NULL;
 
 	int result = OSCAP_ERROR;
+
+	if (action->remediate && _system_is_in_bootc_mode()) {
+		fprintf(stderr,
+			"Detected running Image Mode operating system. OpenSCAP can't "
+			"perform remediation of this system because majority of the "
+			"system is read-only. Please apply remediation during bootable "
+			"container image build using 'oscap-im' instead.\n");
+		return result;
+	}
+
 #if defined(HAVE_SYSLOG_H)
 	int priority = LOG_NOTICE;
 
@@ -641,6 +755,8 @@ int app_evaluate_xccdf(const struct oscap_action *action)
 		}
 		xccdf_session_set_reference_filter(session, action->reference);
 	}
+	if (action->show_rule_details)
+		xccdf_session_set_show_rule_details(session, true);
 
 	if (xccdf_session_load(session) != 0)
 		goto cleanup;
@@ -793,6 +909,14 @@ int app_xccdf_remediate(const struct oscap_action *action)
 {
 	struct xccdf_session *session = NULL;
 	int result = OSCAP_ERROR;
+	if (_system_is_in_bootc_mode()) {
+		fprintf(stderr,
+			"Detected running Image Mode operating system. OpenSCAP can't "
+			"perform remediation of this system because majority of the "
+			"system is read-only. Please apply remediation during bootable "
+			"container image build using 'oscap-im' instead.\n");
+		return result;
+	}
 	session = xccdf_session_new(action->f_xccdf);
 	if (session == NULL)
 		goto cleanup;
@@ -1205,7 +1329,9 @@ enum oval_opt {
     XCCDF_OPT_RESULT_ID = 'i',
 	XCCDF_OPT_FIX_TYPE,
 	XCCDF_OPT_LOCAL_FILES,
-	XCCDF_OPT_REFERENCE
+	XCCDF_OPT_REFERENCE,
+	XCCDF_OPT_VERBOSE,
+	XCCDF_OPT_VERBOSE_LOG_FILE
 };
 
 bool getopt_xccdf(int argc, char **argv, struct oscap_action *action)
@@ -1237,6 +1363,8 @@ bool getopt_xccdf(int argc, char **argv, struct oscap_action *action)
 		{"fix-type", required_argument, NULL, XCCDF_OPT_FIX_TYPE},
 		{"local-files", required_argument, NULL, XCCDF_OPT_LOCAL_FILES},
 		{"reference", required_argument, NULL, XCCDF_OPT_REFERENCE},
+		{"verbose", required_argument, NULL, XCCDF_OPT_VERBOSE},
+		{"verbose-log-file", required_argument, NULL, XCCDF_OPT_VERBOSE_LOG_FILE},
 	// flags
 		{"force",		no_argument, &action->force, 1},
 		{"oval-results",	no_argument, &action->oval_results, 1},
@@ -1254,6 +1382,7 @@ bool getopt_xccdf(int argc, char **argv, struct oscap_action *action)
 		{"without-syschar",     no_argument, &action->without_sys_chars, 1},
 		{"thin-results",        no_argument, &action->thin_results, 1},
 		{"raw",                 no_argument, &action->raw, 1},
+		{"show-rule-details", no_argument, &action->show_rule_details, 1},
 	// end
 		{0, 0, 0, 0}
 	};
@@ -1298,6 +1427,12 @@ bool getopt_xccdf(int argc, char **argv, struct oscap_action *action)
 		case XCCDF_OPT_REFERENCE:
 			action->reference = optarg;
 			break;
+		case XCCDF_OPT_VERBOSE:
+			action->verbosity_level = optarg;
+			break;
+		case XCCDF_OPT_VERBOSE_LOG_FILE:
+			action->f_verbose_log = optarg;
+			break;
 		case 0: break;
 		default: return oscap_module_usage(action->module, stderr, NULL);
 		}
